package org.apache.ws.jaxme.generator.sg.impl;

import org.apache.ws.jaxme.generator.Generator;
import org.apache.ws.jaxme.generator.sg.Context;
import org.apache.ws.jaxme.generator.sg.Facet;
import org.apache.ws.jaxme.generator.sg.GroupSG;
import org.apache.ws.jaxme.generator.sg.ObjectSG;
import org.apache.ws.jaxme.generator.sg.SGFactory;
import org.apache.ws.jaxme.generator.sg.SGFactoryChain;
import org.apache.ws.jaxme.generator.sg.SchemaSG;
import org.apache.ws.jaxme.generator.sg.TypeSG;
import org.apache.ws.jaxme.xs.XSAny;
import org.apache.ws.jaxme.xs.XSElement;
import org.apache.ws.jaxme.xs.XSEnumeration;
import org.apache.ws.jaxme.xs.XSGroup;
import org.apache.ws.jaxme.xs.XSObjectFactory;
import org.apache.ws.jaxme.xs.XSParser;
import org.apache.ws.jaxme.xs.XSSchema;
import org.apache.ws.jaxme.xs.XSType;
import org.apache.ws.jaxme.xs.xml.XsObjectFactory;
import org.apache.ws.jaxme.xs.xml.XsQName;
import org.xml.sax.SAXException;


public class SGFactoryImpl implements SGFactory {
  private SGFactoryChain backingObject;


  public SGFactoryImpl(SGFactoryChain o) {
    if (o == null) {
      throw new NullPointerException("The supplied object must not be null.");
    }
    backingObject = o;
  }

  public Generator getGenerator() {
    return backingObject.getGenerator(this);
  }

  public void init() {
    backingObject.init(this);
  }

  public XsObjectFactory newXsObjectFactory() throws SAXException {
    return backingObject.newXsObjectFactory(this);
  }

  public XSObjectFactory newXSObjectFactory() throws SAXException {
    return backingObject.newXSObjectFactory(this);
  }

  public Object newSchemaSG(XSSchema pSchema) throws SAXException {
    return backingObject.newSchemaSG(this, pSchema);
  }

  public SchemaSG getSchemaSG(XSSchema pSchema) throws SAXException {
    return backingObject.getSchemaSG(this, pSchema);
  }

  public SchemaSG getSchemaSG() {
    return backingObject.getSchemaSG(this);
  }

  public Object newObjectSG(XSElement pElement) throws SAXException {
    return backingObject.newObjectSG(this, pElement);
  }

  public ObjectSG getObjectSG(XSElement pElement) throws SAXException {
    return backingObject.getObjectSG(this, pElement);
  }

  public Object newObjectSG(XSElement pElement, Context pContext) throws SAXException {
    return backingObject.newObjectSG(this, pElement, pContext);
  }

  public ObjectSG getObjectSG(XSElement pElement, Context pContext) throws SAXException {
    return backingObject.getObjectSG(this, pElement, pContext);
  }

  public Object newObjectSG(XSAny any) {
    return backingObject.newObjectSG(this, any);
  }

  public ObjectSG getObjectSG(XSAny pWildcard, Context pContext) throws SAXException {
    return backingObject.getObjectSG(this, pWildcard, pContext);
  }

  public GroupSG newGroupSG(XSGroup pGroup) throws SAXException {
    return backingObject.newGroupSG(this, pGroup);
  }

  public GroupSG getGroupSG(XSGroup pGroup) throws SAXException {
    return backingObject.getGroupSG(this, pGroup);
  }

  public GroupSG newGroupSG(XSGroup pGroup, Context pClassContext) throws SAXException {
    return backingObject.newGroupSG(this, pGroup, pClassContext);
  }

  public GroupSG getGroupSG(XSGroup pGroup, Context pClassContext) throws SAXException {
    return backingObject.getGroupSG(this, pGroup, pClassContext);
  }

  public Object newTypeSG(XSType pType) throws SAXException {
    return backingObject.newTypeSG(this, pType);
  }

  public TypeSG getTypeSG(XSType pType) throws SAXException {
    return backingObject.getTypeSG(this, pType);
  }

  public Object newTypeSG(XSType pType, Context pClassContext, XsQName pName) throws SAXException {
    return backingObject.newTypeSG(this, pType, pClassContext, pName);
  }

  public TypeSG getTypeSG(XSType pType, Context pClassContext, XsQName pName) throws SAXException {
    return backingObject.getTypeSG(this, pType, pClassContext, pName);
  }

  public Object newTypeSG(XSType pType, XsQName pName) throws SAXException {
    return backingObject.newTypeSG(this, pType, pName);
  }

  public TypeSG getTypeSG(XSType pType, XsQName pName) throws SAXException {
    return backingObject.getTypeSG(this, pType, pName);
  }

  public XSParser newXSParser() throws SAXException {
    return backingObject.newXSParser(this);
  }

  public Facet newFacet(XSType pType, XSEnumeration[] pEnumerations) throws SAXException {
    return backingObject.newFacet(this, pType, pEnumerations);
  }

  public GroupSG[] getGroups() {
    return backingObject.getGroups(this);
  }

  public TypeSG[] getTypes() {
    return backingObject.getTypes(this);
  }

  public ObjectSG[] getObjects() {
    return backingObject.getObjects(this);
  }

  public SGFactoryChain getHeadOfChain() {
    return backingObject;
  }

}
