/*
 * Copyright 2005  The Apache Software Foundation
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.ws.jaxme.generator.sg.impl.ccsg;

import org.apache.ws.jaxme.generator.sg.ComplexTypeSG;
import org.apache.ws.jaxme.js.JavaMethod;
import org.apache.ws.jaxme.js.JavaSource;
import org.xml.sax.SAXException;


/** Interface of an object, which generates instances
 * of {@link org.apache.ws.jaxme.JMXmlSerializer}.
 */
public interface DriverSG {
	/** Returns the class being created.
	 */
	public JavaSource getJavaSource();

	/** Returns the type being serialized.
	 */
	public ComplexTypeSG getType();

	/** Generates the method
	 * {@link org.apache.ws.jaxme.impl.JMSAXDriver#getAttributes(org.apache.ws.jaxme.impl.JMSAXDriverController, Object)}.
	 */
	public JavaMethod newGetAttributesMethod() throws SAXException;

	/** Generates the method
	 * {@link org.apache.ws.jaxme.impl.JMSAXDriver#getPreferredPrefix(String)}.
	 */
	public JavaMethod newGetPreferredPrefixMethod() throws SAXException;

	/** Generates the method
	 * {@link org.apache.ws.jaxme.impl.JMSAXDriver#marshalChilds(org.apache.ws.jaxme.impl.JMSAXDriverController, org.xml.sax.ContentHandler, Object)}.
	 */
	public JavaMethod newMarshalChildsMethod() throws SAXException;

	/** Invokes the various <code>newFooMethod</code>
	 * methods.
	 */
	public void generate() throws SAXException;
}
