/*
 * Copyright 2005  The Apache Software Foundation
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.ws.jaxme.generator.sg.impl.ccsg;

import org.apache.ws.jaxme.generator.sg.ComplexTypeSG;
import org.apache.ws.jaxme.generator.sg.SimpleContentSG;
import org.apache.ws.jaxme.js.JavaMethod;
import org.apache.ws.jaxme.js.JavaQName;
import org.apache.ws.jaxme.js.JavaSource;
import org.apache.ws.jaxme.js.LocalJavaField;
import org.apache.ws.jaxme.js.Parameter;
import org.xml.sax.SAXException;


/** Implementation of
 * {@link org.apache.ws.jaxme.generator.sg.impl.ccsg.DriverSG}
 * for elements with simple content.
 */
public class SimpleContentDriverSG extends EmptyElementDriverSG {
	/** Creates a new instance, generating methods for serialization
	 * of the given type into the given source file.
	 */
	public SimpleContentDriverSG(ComplexTypeSG pType, JavaSource pJs) {
		super(pType, pJs);
	}

	public JavaMethod newMarshalChildsMethod() throws SAXException {
		JavaMethod jm = super.newMarshalChildsMethod();
		Parameter paramElement = getParamElement();
		Parameter paramController = getParamController();
		JavaQName elementInterface = ctSG.getClassContext().getXMLInterfaceName();
	    LocalJavaField element = jm.newJavaField(elementInterface);
	    element.addLine("(", elementInterface, ") ", paramElement);
	    LocalJavaField chars = jm.newJavaField(String.class);
	    SimpleContentSG simpleContent = ctSG.getSimpleContentSG();
	    Object value = simpleContent.getPropertySG().getValue(element);
	    chars.addLine(simpleContent.getContentTypeSG().getSimpleTypeSG().getCastToString(jm, value, paramController));
        jm.addIf(chars, " != null  &&  ", chars, ".length() > 0");
	    LocalJavaField charArray = jm.newJavaField(char[].class);
	    charArray.addLine(chars, ".toCharArray()");
	    jm.addLine(paramController, ".getTarget().characters(", charArray, ", 0, ", charArray, ".length);");
        jm.addEndIf();
	    return jm;
	}
}
