package org.apache.ws.jaxme.junit;

import java.io.StringWriter;

import javax.xml.bind.JAXBContext;

import magoffin.matt.ieat.domain.impl.IngredientImpl;
import net.dspc.commons.activitymodel.TransmissionData;
import net.dspc.commons.activitymodel2.Body1;
import net.dspc.commons.activitymodel2.Body2;
import net.dspc.commons.activitymodel2.ObjectFactory;

import org.apache.ws.jaxme.test.misc.types.Row;
import org.apache.ws.jaxme.test.misc.types.impl.RowImpl;
import org.apache.ws.jaxme.test.misc.xsimport.a.Outer;


/** Some excerpts from Jira bug reports.
 */
public class JiraTest extends BaseTestCase {
	/** Creates a new instance with the given name.
	 */
    public JiraTest(String pName) {
    	super(pName);
    }

    /** Test for <a href="http://issues.apache.org/jira/browse/JAXME-10">JAXME-10</a>
     */
    public void testJaxMe10() throws Exception {
		final String input1 =
            "<TransmissionData mail-id=\"mail456.test-account@dspc.net\" xmlns=\"http://commons.dspc.net/activitymodel\">\n" +
            "  <sender account-id=\"test-account@dspc.net\" name=\"Joe Tester\">\n" +
            "    <e-mail>test-account@bluprints.com</e-mail>\n" +
            "    <phone>0793041414141</phone>\n" +
            "    <crypt-signature/>\n" +
            "  </sender>\n" +
            "  <recipient account-id=\"hvendelbo@bluprints.com\" name=\"Henrik Vendelbo\"/>\n" +
            "</TransmissionData>";
        unmarshalMarshalUnmarshal(TransmissionData.class, input1);
        final String input2 =
            "<TransmissionData mail-id=\"mail456.test-account@dspc.net\" xmlns=\"http://commons.dspc.net/activitymodel\">\n" +
            "  <sender account-id=\"test-account@dspc.net\" name=\"Joe Tester\">\n" +
            "    <e-mail>test-account@bluprints.com</e-mail>\n" +
            "    <phone>0793041414141</phone>\n" +
            "  </sender>\n" +
            "  <recipient account-id=\"hvendelbo@bluprints.com\" name=\"Henrik Vendelbo\"/>\n" +
            "</TransmissionData>";
        unmarshalMarshalUnmarshal(TransmissionData.class, input2);
        final String input3 =
            "<TransmissionData mail-id=\"mail456.test-account@dspc.net\" xmlns=\"http://commons.dspc.net/activitymodel\">\n" +
            "  <sender account-id=\"test-account@dspc.net\" name=\"Joe Tester\"/>\n" +
            "  <recipient account-id=\"hvendelbo@bluprints.com\" name=\"Henrik Vendelbo\"/>\n" +
            "</TransmissionData>";
        unmarshalMarshalUnmarshal(TransmissionData.class, input2);
    }

    /** Test for <a href="http://issues.apache.org/jira/browse/JAXME-12">JAXME-12</a>
     */
    public void testJaxMe12() throws Exception {
        final String[] inputs = new String[]{
            "<Body1 text-template=\"y\" html-template=\"x\" xmlns=\"http://commons.dspc.net/activitymodel2\"/>",
            "<Body1 text-template=\"y\" xmlns=\"http://commons.dspc.net/activitymodel2\"/>",
            "<Body1 html-template=\"x\" xmlns=\"http://commons.dspc.net/activitymodel2\"/>",
            "<Body1 xmlns=\"http://commons.dspc.net/activitymodel2\"/>",
            "<Body2 xmlns=\"http://commons.dspc.net/activitymodel2\"/>"
        };
        for (int i = 0;  i < inputs.length;  i++) {
        	unmarshalMarshalUnmarshal(Body1.class, inputs[i]);
        }
        Body1 body1 = new ObjectFactory().createBody1();
        body1.setValue("ok");
        Body2 body2 = new ObjectFactory().createBody2();
        body2.setValue("ok");
    }

    /** Test for <a href="http://issues.apache.org/jira/browse/JAXME-38">JAXME-38</a>
     */
    public void testJaxMe38() throws Exception {
    	RowImpl emptyRow = new RowImpl();
        final String xml = "<ex:row xmlns:ex=\"" + getNamespaceURI(emptyRow) + "\">\n" +
        "  <ex:cell id=\"1\">a</ex:cell>\n" +
        "</ex:row>";
        unmarshalMarshalUnmarshal(Row.class, xml);
    }

    /** Test for <a href="http://issues.apache.org/jira/browse/JAXME-39">JAXME-39</a>
     */
    public void testJaxMe39() throws Exception {
    	IngredientImpl ingredientImpl = new IngredientImpl();
        assertTrue(!ingredientImpl.isSetIngredientId());
        ingredientImpl.setIngredientId(new Integer(0));
        assertTrue(ingredientImpl.isSetIngredientId());
    }

    /** Test for <a href="http://issues.apache.org/jira/browse/JAXME-45">JAXME-45</a>
     */
    public void testJaxMe45() throws Exception {
    	Outer outer = new org.apache.ws.jaxme.test.misc.xsimport.a.ObjectFactory().createOuter();
        outer.setInner1("ok");
        outer.setInner2(0);
        JAXBContext context = JAXBContext.newInstance("org.apache.ws.jaxme.test.misc.xsimport.a");
        StringWriter sw = new StringWriter();
        context.createMarshaller().marshal(outer, sw);
        assertEquals("<?xml version='1.0' encoding='UTF-8'?>\n" +
                     "<a:outer xmlns:a=\"http://ws.apache.org/jaxme/test/misc/xsimport/a\">\n" +
                     "  <b:inner1 xmlns:b=\"http://ws.apache.org/jaxme/test/misc/xsimport/b\">ok</b:inner1>\n" +
                     "  <b:inner2 xmlns:b=\"http://ws.apache.org/jaxme/test/misc/xsimport/b\">0</b:inner2>\n" +
                     "</a:outer>", sw.toString());
    }
}
